from random import randint, choice

class ProblemGenerators(object):
    """
    Abstract base class for problem generator factories.

    """
    def __init__(self):
        self.func_map = {"+": lambda a,b: a + b,
                         "-": lambda a,b: a - b,
                         "*": lambda a,b: a * b,
                         "^": lambda a,b: a ** b,}

        # ATTN This is temporary
        self.rend_map = {"+": lambda a,b: "{kerning -5}%d + %d{kerning 0} " % (a,b),
                         "-": lambda a,b: "{kerning -5}%d - %d{kerning 0} " % (a,b),
                         "*": lambda a,b: "{kerning -5}%d {baseline -5}*{baseline 0} %d{kerning 0} " % (a,b),
                         "^": lambda a,b: "{kerning 0}%d{baseline 20}{font_size 24}%d{baseline 0}{font_size 36}{kerning 0} " % (a,b),}


    def get(self):
        return self._get().next()

    def solve(self, prob):
        return self.func_map[prob[0]](*prob[1:])

    def render(self, prob):
        return self.rend_map[prob[0]](*prob[1:])

    
def rand_ints_in_union(n, range_list):
    """
    Returns uniform random integers from a union
    of intervals.

    N is the number of integers to return, collected
    into a list.

    RANGE_LIST is a list of tuples of the form (a,b)
    with a <= b.

    """
    return [randint(*choice(range_list)) for i in range(n)]


class IntegerAdditionProblem(ProblemGenerators):
    """
    Generates integer addition problems with fixed
    of digits and optional sign.

    """
    def __init__(self, range_list):
        """
        Return factory for simple integer addition problems
        """
        super(IntegerAdditionProblem, self).__init__()

        def next():
            while True:
                x, y = rand_ints_in_union(2, range_list)
            
                yield ("+", x, y)

        self._get = next

class IntegerSubtractionProblem(ProblemGenerators):
    """
    Generates integer subtraction problems with fixed
    of digits and optional sign.

    """

    def __init__(self, range_list):
        """
        Return factory for simple integer subtraction problems
        """
        super(IntegerSubtractionProblem, self).__init__()

        def next():
            while True:
                x, y = rand_ints_in_union(2, range_list)
            
                yield ("-", x, y)

        self._get = next

class IntegerMultiplicationProblem(ProblemGenerators):
    """
    Generates integer multiplication problems with fixed
    of digits and optional sign.

    """

    def __init__(self, range_list):
        """
        Return factory for simple integer multiplication problems
        """
        super(IntegerMultiplicationProblem, self).__init__()

        def next():
            while True:
                x, y = rand_ints_in_union(2, range_list)
            
                yield ("*", x, y)

        self._get = next
            
class PowersOfTwo(ProblemGenerators):
    """
    Generates integer multiplication problems with fixed
    of digits and optional sign.

    """

    def __init__(self, minexp, maxexp):
        """
        Return factory for simple integer multiplication problems
        """
        super(PowersOfTwo, self).__init__()

        def next():
            while True:
                y = randint(minexp,maxexp)
            
                yield ("^", 2, y)

        self._get = next

class Squares(ProblemGenerators):
    """
    Generates integer multiplication problems with fixed
    of digits and optional sign.

    """

    def __init__(self, a, b):
        """
        Return factory for simple integer multiplication problems
        """
        super(Squares, self).__init__()

        def next():
            while True:
                x = randint(a,b)
            
                yield ("^", x, 2)

        self._get = next
            
class Cubes(ProblemGenerators):
    """
    Generates integer multiplication problems with fixed
    of digits and optional sign.

    """

    def __init__(self, a, b):
        """
        Return factory for simple integer multiplication problems
        """
        super(Cubes, self).__init__()

        def next():
            while True:
                x = randint(a,b)
            
                yield ("^", x, 3)

        self._get = next
